#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtk.h>
#include <pthread.h>
#include <errno.h>

#include "interface.h"
#include "callbacks.h"
#include "support.h"
#include "utils.h"
#include "tpinterface.h"

PrintInfo* generate_print();

pthread_t   test;
int	    not_initiated = TRUE;

PrintInfo* generate_print()
{
    GtkWidget*	TCP_send_best = NULL;
    GtkWidget*	UDP_send_best = NULL;
    GtkWidget*	TCP_receive_best = NULL;
    GtkWidget*	UDP_receive_best = NULL;
    GtkWidget*	best_load_send = NULL;
    GtkWidget*	best_load_receive = NULL;
    GtkWidget*	test_server = NULL;
    PrintInfo*	print_info = NULL;

    if (my_state == MYSTATE_IDLE && engp)
    {
	print_info = g_malloc(sizeof(PrintInfo));
	g_return_val_if_fail(print_info != NULL, NULL);

	TCP_send_best = lookup_widget(mainwindow, "TCP_send_result");
        UDP_send_best = lookup_widget(mainwindow, "UDP_send_result");
	TCP_receive_best = lookup_widget(mainwindow, "TCP_receive_result");
	UDP_receive_best = lookup_widget(mainwindow, "UDP_receive_result");
	best_load_send = lookup_widget(mainwindow, "total_send");
        best_load_receive = lookup_widget(mainwindow, "total_receive");
	test_server = lookup_widget(mainwindow, "server_entry");

	print_info->tcp_best_send = 
	    gtk_entry_get_text((GtkEntry *) TCP_send_best);
        print_info->tcp_best_receive = 
	    gtk_entry_get_text((GtkEntry *) TCP_receive_best);
	print_info->udp_best_send = 
	    gtk_entry_get_text((GtkEntry *) UDP_send_best);
        print_info->udp_best_receive = 
	    gtk_entry_get_text((GtkEntry *) UDP_receive_best);
	print_info->send_best_load = 
	    gtk_entry_get_text((GtkEntry *) best_load_send);
	print_info->receive_best_load = 
	    gtk_entry_get_text((GtkEntry *) best_load_receive);
	print_info->server =
	    gtk_entry_get_text((GtkEntry *) test_server);
	print_info->start_time = starttime;
	print_info->stop_time = stoptime;
    }

    return print_info;
}

void
on_print_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    PrintInfo*	print_info = NULL;
	menuitem = NULL;
	user_data = NULL;

    print_info = generate_print();
    if (print_info)
	do_print(print_info, FALSE);
    g_free(print_info);
}


void
on_about_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *dialog = NULL;
	menuitem = NULL;
	user_data = NULL;
    
    dialog = create_aboutdialog();
    gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
	dialog = NULL;
}


void
on_help_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    gchar* link = "http://www.internetkvalitetsguide.dk/StreamSpeed_Download.asp";
	menuitem = NULL;
	user_data = NULL;

    activate_url(NULL, link, NULL);
}

int
delete_event			       (GtkMenuItem	*menuitem,
					gpointer	user_data )
{
	menuitem = NULL;
	user_data = NULL;
	
	return FALSE;
}


void
on_quit_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	menuitem = NULL;
	user_data = NULL;

	if (my_state != MYSTATE_IDLE)
	StopContext(engp);
    if (test)
	pthread_cancel(test);
    DeleteContext(engp);
    gtk_main_quit();
}

void
on_preview_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    PrintInfo*	print_info = NULL;
	menuitem = NULL;
	user_data = NULL;

    print_info = generate_print();
    if (print_info)
	do_print(print_info, TRUE);
    g_free(print_info);
}

void
on_mainwindow_show		       (GtkWidget       *widget,
					gpointer         user_data)
{
    const gchar*    msg = NULL;
    GtkWidget*	    dialog = NULL;
    GtkButton*	    button = NULL;
    GtkWidget*	    button_image = NULL;
	widget = NULL;
	user_data = NULL;

    has_server = FALSE;
    button = (GtkButton *) lookup_widget(mainwindow, "start_stop");
    button_image = gtk_image_new_from_stock(
	"gtk-yes",
	GTK_ICON_SIZE_BUTTON);
    gtk_button_set_image(button, button_image);
    if (not_initiated)
    {
	if ((msg = initialize(FALSE)) == NULL)
	{
	    not_initiated = FALSE;
	    my_state = MYSTATE_GETSERVERLIST;
	    substate = 0;
	    tptest(NULL);
	    my_state = MYSTATE_IDLE;
	}
	else
	{
	    dialog = messagebox(
		_("Initialize"), msg,
		GTK_MESSAGE_INFO, GTK_BUTTONS_OK);
	    gtk_dialog_run((GtkDialog *) dialog);
	    gtk_widget_destroy(dialog);
	}
    }
}

void
on_start_stop_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget*	    dialog = NULL;
    GtkWidget*	    button_image = NULL;
    pthread_attr_t  attr;
	user_data = NULL;

    pthread_attr_init(&attr);
    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);

    if (my_state != MYSTATE_IDLE)
    {
        ClearBestRates(engp);
        update_testing(engp, FALSE);
	update_server_list(engp, FALSE);
        StopContext(engp);
        clear_test_results(FALSE);
	init_progressbar(FALSE);
        my_state = MYSTATE_IDLE;
        gtk_button_set_label(button, _("_START"));
	button_image = gtk_image_new_from_stock(
		"gtk-yes",
		GTK_ICON_SIZE_BUTTON);
	gtk_button_set_image(button, button_image);
    }

    else
    {
	my_state = MYSTATE_STDTEST;
	substate = 0;
	ClearBestRates(engp);
	clear_test_results(FALSE);
	gtk_button_set_label(button, _("_STOP"));
	button_image = gtk_image_new_from_stock(
		"gtk-stop",
		GTK_ICON_SIZE_BUTTON);
	gtk_button_set_image(button, button_image);
	if (pthread_create(&test, &attr, tptest, (void *) NULL))
	{
	    dialog = messagebox(
		_("Run test"), _("Unable to create new thread to run test"),
		GTK_MESSAGE_INFO, GTK_BUTTONS_OK);
		gtk_dialog_run((GtkDialog *) dialog);
		gtk_widget_destroy(dialog);
	}
    }
}

void
on_lookup_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget*	    hostname_entry = NULL;
    GtkWidget*	    port_entry = NULL;
    GtkWidget*	    master_check = NULL;
    GtkWidget*	    test_servers = NULL;
    GtkWidget*	    master_server = NULL;
    GtkWidget*	    test_server = NULL;
    GtkTextBuffer*  text_buffer_cur = NULL;
    GtkTextBuffer*  text_buffer_new = NULL;
    GtkTextIter	    begin, end;
    GtkWidget*	    dialog = NULL;
    const gchar*    hostname = NULL;
    long int	    port;
    int		    res;
    gchar*	    error = NULL;
    gchar*	    str = NULL;
    const gchar*    empty = "";
	button = NULL;
	user_data = NULL;

    ClearServerList(engp);
    hostname_entry = lookup_widget(mainwindow, "new_master_entry");
    hostname = gtk_entry_get_text((GtkEntry *) hostname_entry);
    if (!hostname || strlen(hostname) == 0)
    {
	dialog = messagebox(
		_("New Host"),
		_("Hostname cannot be empty"),
		GTK_MESSAGE_ERROR, GTK_BUTTONS_OK);
	gtk_dialog_run((GtkDialog *) dialog);
	gtk_widget_destroy(dialog);
	return;
    }
    port_entry = lookup_widget(mainwindow, "master_port_entry");
    errno = 0;
    port = strtol(gtk_entry_get_text((GtkEntry *) port_entry), NULL, 10);
    if (errno != 0 || port == 0)
    {
	dialog = messagebox(
		_("New Host"),
		_("0 < Port < 65535 was not satisfied"),
		GTK_MESSAGE_ERROR, GTK_BUTTONS_OK);
	gtk_dialog_run((GtkDialog *) dialog);
	gtk_widget_destroy(dialog);
	return;
    }
    master_check = lookup_widget(mainwindow, "checkbutton1");
    strcpy(engp->hostName, hostname);
    engp->hostCtrlPort = (USHORT) port;
    master_mode = gtk_toggle_button_get_active((GtkToggleButton *) master_check);

    if (name_lookup(engp) == NOSUCHHOST)
    {
	dialog = messagebox(
		_("New server"),
		_("Host unknown"),
		GTK_MESSAGE_ERROR, GTK_BUTTONS_OK);
	gtk_dialog_run((GtkDialog *) dialog);
	gtk_widget_destroy(dialog);
	return;
    }
    if (master_mode)
    {
	engp->tpMode = CLM_NONE;
	while (TRUE)
	{
	    res = get_server_list(engp);
	    if (res == 0)
	    {
		if (engp->active != 0)
		    continue;
		else
		{
		    if (engp->numServers == 0)
			res = -1;
		    break;
		}
	    }
	    else
		break;
	}
	switch (res)
	{
	    case -1:
		error = g_strdup(_("Failed to get server list"));
		break;
	    case QUERYMASTERFAIL:
		error = g_strdup(_("Query master server failed"));
		break;
	    case QUERYMASTERINITFAIL:
		error = g_strdup(_("Could not connect to master server"));
		break;
	}
	if (error)
	{
	    dialog = messagebox(
		_("New server"),
		error,
		GTK_MESSAGE_ERROR, GTK_BUTTONS_OK);
	    gtk_dialog_run((GtkDialog *) dialog);
	    gtk_widget_destroy(dialog);
	    return;
	}
	else
	    server_selected(FALSE);
    }
    test_servers = lookup_widget(mainwindow, "new_test_servers");
    text_buffer_new = gtk_text_view_get_buffer((GtkTextView *) test_servers);
    master_server = lookup_widget(mainwindow, "master_entry");
    test_server = lookup_widget(mainwindow, "server_entry");
    test_servers = lookup_widget(mainwindow, "cur_test_servers");
    text_buffer_cur = gtk_text_view_get_buffer((GtkTextView *) test_servers);
    if (!master_mode)
    {
	gtk_text_buffer_set_text(text_buffer_new, empty, strlen(empty));
	gtk_entry_set_text((GtkEntry *) master_server, _("NONE"));
	gtk_text_buffer_set_text(text_buffer_cur, empty, strlen(empty));
	gtk_entry_set_text((GtkEntry *) test_server,
		g_strdup_printf("%s:%d", hostname, (USHORT) port));
    }
    else
    {
	gtk_text_buffer_get_start_iter(text_buffer_cur, &begin);
	gtk_text_buffer_get_end_iter(text_buffer_cur, &end);
	str = gtk_text_buffer_get_text(text_buffer_cur, &begin, &end, FALSE);
	gtk_text_buffer_set_text(text_buffer_new, str, strlen(str));
	gtk_entry_set_text((GtkEntry *) master_server,
		g_strdup_printf("%s:%d", hostname, (USHORT) port));
	gtk_entry_set_text((GtkEntry *) test_server, empty);
    }
    my_state = MYSTATE_IDLE;
    substate = 0;
}

void
on_notebook_switch_page			(GtkNotebook     *notebook,
					 GtkNotebookPage *page,
					 guint            page_num,
					 gpointer         user_data)
{
    GtkWidget*	    hostname_entry = NULL;
    GtkWidget*	    port_entry = NULL;
    GtkWidget*	    test_servers = NULL;
    GtkWidget*	    master_check = NULL;
    /*GtkNotebook*    notebook = NULL;*/
    GtkTextBuffer*  text_buffer_new = NULL;
    const gchar*    empty = "";
	page = NULL;
	page_num = 0;
	user_data = NULL;
	
    /*notebook = (GtkNotebook *) lookup_widget(mainwindow, "notebook1");*/
    if (gtk_notebook_get_current_page(notebook) == 1)
    {
	hostname_entry = lookup_widget(mainwindow, "new_master_entry");
	gtk_entry_set_text((GtkEntry *) hostname_entry, empty);
	port_entry = lookup_widget(mainwindow, "master_port_entry");
	gtk_entry_set_text((GtkEntry *) port_entry, empty);
	test_servers = lookup_widget(mainwindow, "new_test_servers");
	text_buffer_new = gtk_text_view_get_buffer((GtkTextView *) test_servers);
	gtk_text_buffer_set_text(text_buffer_new, empty, strlen(empty));
	master_check = lookup_widget(mainwindow, "checkbutton1");
	gtk_toggle_button_set_active((GtkToggleButton *) master_check, TRUE);
    }
}

